@@ -6,9 +6,9 @@ import numba
 import numpy as np
 from numba import njit, prange
 
-from .numerics import _calc_mean_shrunken_residuals
+from .numerics import _calc_mean_shrunken_residuals, _count_n_cells, _count_n_cpg
 from .smooth import _load_smoothed_chrom
-from .utils import _check_data_dir, _load_chrom_mat, echo, secho
+from .utils import _check_data_dir, _get_filepath, _load_chrom_mat, echo, secho
 
 # ignore division by 0 and division by NaN error
 np.seterr(divide="ignore", invalid="ignore")
@@ -97,7 +97,7 @@ def _move_windows(
     return windows, smoothed_var
 
 
-def scan(data_dir, output, bandwidth, stepsize, var_threshold, threads=-1):
+def scan(data_dir, output, bandwidth, stepsize, var_threshold, min_cells, threads=-1):
     _check_data_dir(data_dir, assert_smoothed=True)
     if threads != -1:
         numba.set_num_threads(threads)
@@ -155,6 +155,15 @@ def scan(data_dir, output, bandwidth, stepsize, var_threshold, threads=-1):
         # for each big merged peak, re-calculate the variance and
         # write it to a file.
         for ps, pe in zip(peak_starts, peak_ends):
+            # get some basic info about the VMR: how many CpGs does it contain,
+            # how many cells have coverage of the region?
+            region_indices = mat.indices[mat.indptr[ps] : mat.indptr[pe + 1]]
+            n_obs_cells = _count_n_cells(region_indices)
+            if n_obs_cells < min_cells:
+                continue  # not enough coverage to report the VMR
+            region_indptr = mat.indptr[ps : pe + 2] - mat.indptr[ps]
+            n_cpg = _count_n_cpg(region_indptr)
+            # calculate variance for the whole peak
             peak_var = np.nanvar(
                 _calc_mean_shrunken_residuals(
                     mat.data,
@@ -167,32 +176,25 @@ def scan(data_dir, output, bandwidth, stepsize, var_threshold, threads=-1):
                     chrom_len,
                 )
             )
-            bed_entry = f"{chrom}\t{ps}\t{pe}\t{peak_var}\n"
+            # write a row to the output bed file
+            bed_entry = f"{chrom}\t{ps}\t{pe}\t{peak_var}\t{n_cpg}\t{n_obs_cells}\n"
             output.write(bed_entry)
         if len(peak_starts) > 0:
             secho(
-                f"Found {len(peak_starts)} variable regions on chromosome {chrom}.",
+                f"Found {len(peak_starts)} variably methylated regions on "
+                f"chromosome {chrom}.",
                 fg="green",
             )
         else:
             secho(
-                f"Found no variable regions on chromosome {chrom}.",
+                f"Found no variably methylated regions on chromosome {chrom}.",
                 fg="red",
             )
+    echo(
+        f"\nWrote VMRs with sequencing coverage in at least {min_cells} cells "
+        f"to {_get_filepath(output)}\n"
+        "The columns in this file correspond to:\n"
+        "chromosome, VMR start, VMR end, variance, # of CpG sites, "
+        "# of cells with coverage in the VMR"
+    )
     return
-
-
-# currently not needed but could be useful:
-# @njit
-# def _count_n_cells(region_indices):
-#     """
-#     Count the total number of CpGs in a region, based on CSR matrix indices.
-#     Only CpGs that have coverage in at least 1 cell are counted.
-#     """
-#     seen_cells = set()
-#     n_cells = 0
-#     for cell_idx in region_indices:
-#         if cell_idx not in seen_cells:
-#             seen_cells.add(cell_idx)
-#             n_cells += 1
-#     return n_cells
